@@ -1,12 +1,26 @@
 package Business::OnlinePayment::SagePay;
-
+{
+  $Business::OnlinePayment::SagePay::VERSION = '0.13';
+}
+# vim: ts=2 sts=2 sw=2 :
+  
 use strict;
+use warnings;
+
+use base qw(Business::OnlinePayment);
+
 use Carp;
 use Net::SSLeay qw(make_form post_https);
-use base qw(Business::OnlinePayment);
-use Data::Dumper;
+use Devel::Dwarn;
+use Exporter 'import';
 
-our $VERSION = '0.12';
+use constant {
+  SAGEPAY_STATUS_OK               => 'OK',
+  SAGEPAY_STATUS_AUTHENTICATED    => 'AUTHENTICATED',
+  SAGEPAY_STATUS_REGISTERED       => 'REGISTERED',
+  SAGEPAY_STATUS_3DSECURE         => '3DAUTH',
+  SAGEPAY_STATUS_PAYPAL_REDIRECT  => 'PPREDIRECT',
+};
 
 # CARD TYPE MAP
 
@@ -17,16 +31,21 @@ my %card_type = (
   'visa electron' => 'UKE',
   'visa debit' => 'DELTA',
   'mastercard' => 'MC',
+  'mastercard debit' => 'MC',
   'maestro' => 'MAESTRO',
   'international maestro' => 'MAESTRO',
   'switch' => 'MAESTRO',
-  'switch solo' => 'SOLO',
+'switch solo' => 'SOLO',
   'solo' => 'SOLO',
   'diners club' => 'DINERS',
   'jcb' => 'JCB',
+  'paypal' => 'PAYPAL',
 );
 
 my $status = {
+  TIMEOUT => 'There was a problem communicating with the payment server, please try later',
+  UNKNOWN => 'There was an unknown problem taking your payment. Please try again',
+  '3D_PASS' => 'Your card failed the password check.',
   2000 => 'Your card was declined by the bank.',
   5013 => 'Your card has expired.',
   3078 => 'Your e-mail was invalid.',
@@ -44,14 +63,16 @@ my $status = {
   5018 => "Your card security number was the incorrect length. This is normally the last 3 digits on the back of your card",
   3130 => "Your state was incorrect. Please use the standard two character state code",
   3068 => "Your card type is not supported by this vendor. Please try a different card",
+  5055 => "Your postcode had incorrect characters. Please re-enter",
+  3055 => "Your card was not recognised. Please try another",
 };
 
 #ACTION MAP
 my %action = (
-    'normal authorization'    => 'PAYMENT',       # Take Payment now
-    'authorization only'      => 'AUTHENTICATE',  # Store details at sagepay with 3D+address check
-    'post authorization'      => 'AUTHORISE',	    # Post-Auth
-    'refund'                  => 'REFUND',
+  'normal authorization' => 'PAYMENT',
+  'authorization only'   => 'AUTHENTICATE',
+  'post authorization'   => 'AUTHORISE',
+  'refund'               => 'REFUND',
 );
 
 my %servers = (
@@ -62,6 +83,7 @@ my %servers = (
     authorise => '/gateway/service/authorise.vsp',
     refund => '/gateway/service/refund.vsp',
     cancel => '/gateway/service/cancel.vsp',
+    complete => '/gateway/service/complete.vsp',
     port => 443,
   },
   test => {
@@ -71,6 +93,7 @@ my %servers = (
     authorise => '/gateway/service/authorise.vsp',
     refund => '/gateway/service/refund.vsp',
     cancel => '/gateway/service/cancel.vsp',
+    complete => '/gateway/service/complete.vsp',
     port => 443,
   },
   simulator => {
@@ -80,6 +103,7 @@ my %servers = (
     authorise => '/Simulator/VSPServerGateway.asp?service=VendorAuthoriseTx ',
     refund => '/Simulator/VSPServerGateway.asp?service=VendorRefundTx ',
     cancel => '/Simulator/VSPServerGateway.asp?service=VendorCancelTx',
+    complete => '/Simulator/paypalcomplete.asp',
     port => 443,
   },
   timeout => {
@@ -111,12 +135,14 @@ sub set_server {
 sub set_defaults {
   my $self = shift;
   $self->set_server('live');
-
-  $self->build_subs(qw/protocol currency cvv2_response postcode_response error_code
-     require_3d forward_to invoice_number authentication_key pareq cross_reference callback/);
+  $self->build_subs(qw/
+    protocol currency cvv2_response postcode_response address_response error_code require_3d require_paypal
+    forward_to invoice_number authentication_key authorization_code pareq cross_reference callback encoded_3d_result
+  /);
   $self->protocol('2.23');
   $self->currency('GBP');
   $self->require_3d(0);
+  $self->require_paypal(0);
 }
 
 sub do_remap {
@@ -138,14 +164,15 @@ sub format_amount {
 }
 
 sub submit_3d {
-	my $self = shift;
-        my %content = $self->content;
-        my %post_data = (
-          ( map { $_ => $content{$_} } qw(login password) ),
-          MD    => $content{'cross_reference'},
-          PaRes => $content{'pares'},
-        );
-  $self->set_server($ENV{'SAGEPAY_F_SIMULATOR'} ? 'simulator' : 'test') if $self->test_transaction;
+  my $self = shift;
+  my %content = $self->content;
+  my %post_data = (
+    ( map { $_ => $content{$_} } qw(login password) ),
+    MD    => $content{'cross_reference'},
+    PaRes => $content{'pares'},
+  );
+  $self->set_server($ENV{'SAGEPAY_F_SIMULATOR'} ? 'simulator' : 'test') 
+    if $self->test_transaction;
   my ($page, $response, %headers) = 
     post_https(
       $self->server,
@@ -155,7 +182,7 @@ sub submit_3d {
       make_form(%post_data)
     );
   unless ($page) {
-    $self->error_message('There was a problem communicating with the payment server, please try later');
+    $self->error_message($status->{TIMEOUT});
     return;
   }
 
@@ -163,21 +190,23 @@ sub submit_3d {
 
   if($ENV{'SAGEPAY_DEBUG'}) {
     warn "3DSecure:";
-    warn Dumper($rf);
+    Dwarn $rf;
   }
 
   $self->server_response($rf);
   $self->result_code($rf->{'Status'});
   $self->authentication_key($rf->{'SecurityKey'});
   $self->authorization($rf->{'VPSTxId'});
+  $self->authorization_code($rf->{'TxAuthNo'});
+  $self->encoded_3d_result($rf->{'CAVV'});
 
   unless(
-    $self->is_success($rf->{'Status'} eq 'OK'
-    || $rf->{'Status'} eq 'AUTHENTICATED' 
+    ($self->is_success($rf->{'Status'} eq SAGEPAY_STATUS_OK) || 
+    ($rf->{'Status'} eq SAGEPAY_STATUS_AUTHENTICATED)
     ?  1 : 0)) {
-    $self->error_message('Your card failed the password check.');
+    $self->error_message($status->{'3D_PASS'});
     if($ENV{'SAGEPAY_DEBUG_ERROR_ONLY'}) {
-      warn Dumper($rf);
+      Dwarn $rf;
     }
     return 0;
   } else{
@@ -185,12 +214,89 @@ sub submit_3d {
   }
 }
 
+sub submit_paypal {
+  my $self = shift;
+
+  my $content = $self->sanitised_content;
+
+  my %field_mapping = (
+    VpsProtocol => \($self->protocol),
+    Vendor      => \($self->vendor),
+    VPSTxId     => 'authentication_id',
+    Amount      => 'amount',
+  );
+
+  my %post_data = (
+    $self->do_remap($content, %field_mapping),
+    TxType  => 'COMPLETE',
+    Accept  => 'YES',
+  );
+
+  if($ENV{'SAGEPAY_DEBUG'}) {
+    warn "PayPal complete Form:";
+    Dwarn {
+      %post_data, 
+    };
+  }
+
+  $self->set_server($ENV{'SAGEPAY_F_SIMULATOR'} ? 'simulator' : 'test') 
+    if $self->test_transaction;
+
+  $self->path( $servers{ $self->{'_server'} }->{'complete'} );
+
+  my ($page, $response, %headers) = 
+    post_https(
+      $self->server,
+      $self->port,
+      $self->path,
+      undef,
+      make_form(%post_data)
+    );
+
+  unless ($page) {
+    $self->error_message($status->{TIMEOUT});
+    $self->is_success(0);
+    return;
+  }
+
+  my $rf = $self->_parse_response($page);
+
+  if($ENV{'SAGEPAY_DEBUG'}) {
+    warn "PayPal:";
+    Dwarn $rf;
+  }
+
+  $self->server_response($rf);
+  $self->result_code($rf->{'Status'});
+  $self->authorization($rf->{'VPSTxId'});
+  $self->authentication_key($rf->{'SecurityKey'});
+  $self->authorization_code($rf->{'TxAuthNo'});
+
+  unless($self->is_success(
+    $rf->{'Status'} eq SAGEPAY_STATUS_OK ||
+    $rf->{'Status'} eq SAGEPAY_STATUS_AUTHENTICATED 
+    ?  1 : 0)
+  ) {
+    my $code = substr $rf->{'StatusDetail'}, 0 ,4;
+
+    if($ENV{'SAGEPAY_DEBUG_ERROR_ONLY'}) {
+      Dwarn $rf;
+    }
+
+    $self->error_code($code);
+    $self->error_message($status->{$code} || $status->{UNKNOWN});
+
+    return 0;
+  }
+  else {
+    return 1;
+  }
+}
+
 
 sub void_action { #void authorization
   my $self = shift;
-  croak "Need vendor ID"
-    unless defined $self->vendor;
-  $self->set_server($ENV{'SAGEPAY_F_SIMULATOR'} ? 'simulator' : 'test') if $self->test_transaction;
+  $self->initialise;
   my %content = $self->content();
   my %field_mapping = (
     VpsProtocol   => \($self->protocol),
@@ -204,7 +310,7 @@ sub void_action { #void authorization
   $post_data{'TxType'} = 'VOID';
 
   if($ENV{'SAGEPAY_DEBUG'}) {
-    warn Dumper(\%post_data);
+    Dwarn %post_data;
   }
 
   $self->path($servers{$self->{'_server'}}->{'cancel'});
@@ -219,7 +325,7 @@ sub void_action { #void authorization
       )
     );
   unless ($page) {
-    $self->error_message('There was a problem communicating with the payment server, please try later');
+    $self->error_message($status->{TIMEOUT});
     $self->is_success(0);
     return;
   }
@@ -228,24 +334,22 @@ sub void_action { #void authorization
 
   if($ENV{'SAGEPAY_DEBUG'}) {
     warn "Cancellation:";
-    warn Dumper($rf);
+    Dwarn $rf;
   }
 
   $self->server_response($rf);
   $self->result_code($rf->{'Status'});
-  unless($self->is_success($rf->{'Status'} eq 'OK'? 1 : 0)) {
+  unless($self->is_success($rf->{'Status'} eq SAGEPAY_STATUS_OK ? 1 : 0)) {
     if($ENV{'SAGEPAY_DEBUG_ERROR_ONLY'}) {
-      warn Dumper($rf);
+      Dwarn $rf;
     }
     $self->error_message($rf->{'StatusDetail'});
   }  
-  }
+}
 
 sub cancel_action { #cancel authentication
   my $self = shift;
-  croak "Need vendor ID"
-    unless defined $self->vendor;
-  $self->set_server($ENV{'SAGEPAY_F_SIMULATOR'} ? 'simulator' : 'test') if $self->test_transaction;
+  $self->initialise;
   my %content = $self->content();
   my %field_mapping = (
     VpsProtocol   => \($self->protocol),
@@ -259,7 +363,7 @@ sub cancel_action { #cancel authentication
   $post_data{'TxType'} = 'CANCEL';
 
   if($ENV{'SAGEPAY_DEBUG'}) {
-    warn Dumper(\%post_data);
+    Dwarn %post_data;
   }
 
   $self->path($servers{$self->{'_server'}}->{'cancel'});
@@ -274,34 +378,39 @@ sub cancel_action { #cancel authentication
       )
     );
   unless ($page) {
-    $self->error_message('There was a problem communicating with the payment server, please try later');
+    $self->error_message($status->{TIMEOUT});
     $self->is_success(0);
-  	return;
+    return;
   }
 
   my $rf = $self->_parse_response($page);
 
   if($ENV{'SAGEPAY_DEBUG'}) {
     warn "Cancellation:";
-    warn Dumper($rf);
+    Dwarn $rf;
   }
 
   $self->server_response($rf);
   $self->result_code($rf->{'Status'});
-  unless($self->is_success($rf->{'Status'} eq 'OK'? 1 : 0)) {
+  unless($self->is_success($rf->{'Status'} eq SAGEPAY_STATUS_OK ? 1 : 0)) {
     if($ENV{'SAGEPAY_DEBUG_ERROR_ONLY'}) {
-      warn Dumper($rf);
+      Dwarn $rf;
     }
     $self->error_message($rf->{'StatusDetail'});
   }  
 }
 
-sub auth_action { 
+sub initialise {
   my $self = shift;
-  my $action = shift;
   croak "Need vendor ID"
     unless defined $self->vendor;
-  $self->set_server($ENV{'SAGEPAY_F_SIMULATOR'} ? 'simulator' : 'test') if $self->test_transaction;
+  $self->set_server($ENV{'SAGEPAY_F_SIMULATOR'} ? 'simulator' : 'test') 
+    if $self->test_transaction;
+}
+
+sub auth_action { 
+  my ($self, $action) = @_;
+  $self->initialise;
   
   my %content = $self->content();
   my %field_mapping = (
@@ -310,7 +419,7 @@ sub auth_action {
     TxType      => \($action{lc $content{'action'}}),
     VendorTxCode=> 'invoice_number',
     Description => 'description',
-    Currency	=> \($self->currency),
+    Currency  => \($self->currency),
     Amount      => \(format_amount($content{'amount'})),
     RelatedVPSTxId => 'parent_auth',
     RelatedVendorTxCode => 'parent_invoice_number',
@@ -319,7 +428,7 @@ sub auth_action {
   my %post_data = $self->do_remap(\%content,%field_mapping);
 
   if($ENV{'SAGEPAY_DEBUG'}) {
-    warn Dumper(\%post_data);
+    Dwarn %post_data;
   }
 
   $self->path($servers{$self->{'_server'}}->{lc $post_data{'TxType'}});
@@ -334,7 +443,7 @@ sub auth_action {
       )
     );
   unless ($page) {
-    $self->error_message('There was a problem communicating with the payment server, please try later');
+    $self->error_message($status->{TIMEOUT});
     $self->is_success(0);
     return;
   }
@@ -343,54 +452,104 @@ sub auth_action {
 
   if($ENV{'SAGEPAY_DEBUG'}) {
     warn "Authorization:";
-    warn Dumper($rf);
+    Dwarn $rf;
   }
 
   $self->server_response($rf);
   $self->result_code($rf->{'Status'});
   $self->authorization($rf->{'VPSTxId'});
-  unless($self->is_success($rf->{'Status'} eq 'OK'? 1 : 0)) {
+  unless($self->is_success($rf->{'Status'} eq SAGEPAY_STATUS_OK ? 1 : 0)) {
     if($ENV{'SAGEPAY_DEBUG_ERROR_ONLY'}) {
-      warn Dumper($rf);
+      Dwarn $rf;
     }
     my $code = substr $rf->{'StatusDetail'}, 0 ,4;
     $self->error_code($code);
-    $self->error_message($status->{$code} || 'There was an unknown problem taking your payment. Please try again');
+    $self->error_message($status->{$code} || $status->{UNKNOWN});
   }
 
 }
 
+sub sanitised_content {
+  my ($self,$content) = @_;
+  my %content = $self->content();
+
+  {
+    no warnings 'uninitialized';
+
+    $content{'expiration'} =~ s#/##g;
+    $content{'startdate'} =~ s#/##g if $content{'startdate'};
+
+    $content{'card_name'} = 
+        $content{'name_on_card'} 
+      || $content{'first_name'} . ' ' . ($content{'last_name'}||"");
+    $content{'customer_name'} = 
+        $content{'customer_name'}
+      || $content{'first_name'} ? 
+            $content{'first_name'} . ' ' . $content{'last_name'} : undef;
+    # new protocol requires first and last name - do some people even have both!?
+    $content{'last_name'} ||= $content{'first_name'}; 
+    $content{'action'} = $action{ lc $content{'action'} };
+    $content{'card_type'} = $card_type{lc $content{'type'}};
+    $content{'amount'} = format_amount($content{'amount'})
+      if $content{'amount'};
+  }
+  
+  return \%content;
+}
+
+sub post_request {
+  my ($self,$type,$data) = @_;
+  
+  $self->path($servers{$self->{'_server'}}->{$type});
+  
+  if($ENV{'SAGEPAY_DEBUG'}) {
+    warn sprintf("Posting to %s:%s%s",
+      $self->server, $self->port, $self->path);
+    Dwarn $data;
+  }
+  
+  my ($page, $response, %headers) = post_https(
+    $self->server,
+    $self->port,
+    $self->path,
+    undef,
+    make_form( %$data )
+  );
+  
+  unless ($page) {
+    $self->error_message($status->{TIMEOUT});
+    $self->is_success(0);
+    return;
+  }
+
+  my $rf = $self->_parse_response($page);
+  $self->server_response($rf);
+  $self->result_code($rf->{'Status'});
+  $self->authorization($rf->{'VPSTxId'});
+  $self->authentication_key($rf->{'SecurityKey'});
+}
+
 sub submit {
   my $self = shift;
-  croak "Need vendor ID"
-    unless defined $self->vendor;
-  $self->set_server($ENV{'SAGEPAY_F_SIMULATOR'} ? 'simulator' : 'test') if $self->test_transaction;
+  $self->initialise;
+  my $content = $self->sanitised_content;
   
-  my %content = $self->content();
-  $content{'expiration'} =~ s#/##g;
-  $content{'startdate'} =~ s#/##g if $content{'startdate'};
-
-  my $card_name = $content{'name_on_card'}||$content{'first_name'} . ' ' . ($content{'last_name'}||"");
-  my $customer_name = $content{'customer_name'}
-  || $content{'first_name'} ? $content{'first_name'} . ' ' . $content{'last_name'} : undef;
-  $content{'last_name'} ||= $content{'first_name'}; # new protocol requires first and last name - do some people even have both!?
   my %field_mapping = (
     VpsProtocol => \($self->protocol),
     Vendor      => \($self->vendor),
-    TxType      => \($action{lc $content{'action'}}),
+    TxType      => 'action',
     VendorTxCode=> 'invoice_number',
     Description => 'description',
     Currency    => \($self->currency),
-    CardHolder  => \($card_name),
+    CardHolder  => 'card_name',
     CardNumber  => 'card_number',
     CV2         => 'cvv2',
-    ExpiryDate	=> 'expiration',
-    StartDate	=> 'startdate',
-    Amount      => \(format_amount($content{'amount'})),
+    ExpiryDate  => 'expiration',
+    StartDate => 'startdate',
+    Amount      => 'amount',
     IssueNumber => 'issue_number',
-    CardType	=> \($card_type{lc $content{'type'}}),
+    CardType  => 'card_type',
     ApplyAVSCV2 => 0,
-
     BillingSurname  => 'last_name',
     BillingFirstnames  => 'first_name',
     BillingAddress1  => 'address',
@@ -407,21 +566,28 @@ sub submit {
     DeliveryCountry => 'country',
     DeliveryState => 'state',
 
-    CustomerName    => \($customer_name),
+    CustomerName    => 'customer_name',
     ContactNumber   => 'telephone',
-    ContactFax		=> 'fax',
-    CustomerEmail	=> 'email',
+    ContactFax    => 'fax',
+    CustomerEmail => 'email',
+
+    PayPalCallbackURL => 'paypal_callback_uri',
   );
 
-  my %post_data = $self->do_remap(\%content,%field_mapping);
+  my %post_data = $self->do_remap($content,%field_mapping);
 
   if($ENV{'SAGEPAY_DEBUG'}) {
     warn "Authentication Form:";
-    warn Dumper({%post_data, CV2 => "XXX", CardNumber => "XXXX XXXX XXXX XXXX"});
+    Dwarn {
+      %post_data, 
+      CV2 => "XXX", 
+      CardNumber => "XXXX XXXX XXXX XXXX"
+    };
   }
 
-  $self->path($servers{$self->{'_server'}}->{'authorise'}) if $post_data{'TxType'} eq 'AUTHORISE';
-  my ($page, $response, %headers) =	post_https(
+  $self->path($servers{$self->{'_server'}}->{'authorise'}) 
+    if $post_data{'TxType'} eq 'AUTHORISE';
+  my ($page, $response, %headers) = post_https(
     $self->server,
     $self->port,
     $self->path,
@@ -431,7 +597,7 @@ sub submit {
     )
   );
   unless ($page) {
-    $self->error_message('There was a problem communicating with the payment server, please try later');
+    $self->error_message($status->{TIMEOUT});
     $self->is_success(0);
     return;
   }
@@ -441,31 +607,47 @@ sub submit {
   $self->result_code($rf->{'Status'});
   $self->authorization($rf->{'VPSTxId'});
   $self->authentication_key($rf->{'SecurityKey'});
+  $self->authorization_code($rf->{'TxAuthNo'});
 
-  if($self->result_code eq '3DAUTH' && $rf->{'3DSecureStatus'} eq 'OK') {
+  if (
+    ($self->result_code eq SAGEPAY_STATUS_3DSECURE) && 
+    ($rf->{'3DSecureStatus'} eq SAGEPAY_STATUS_OK)
+  ) {
     $self->require_3d(1);
     $self->forward_to($rf->{'ACSURL'});
     $self->pareq($rf->{'PAReq'});
     $self->cross_reference($rf->{'MD'});
   }
+
+  if($self->result_code eq SAGEPAY_STATUS_PAYPAL_REDIRECT) {
+    $self->require_paypal(1);
+    $self->forward_to($rf->{'PayPalRedirectURL'});
+  }
+
   $self->cvv2_response($rf->{'CV2Result'});
   $self->postcode_response($rf->{'PostCodeResult'});
+  $self->address_response($rf->{'AddressResult'});
+
   if($ENV{'SAGEPAY_DEBUG'}) {
     warn "Authentication Response:";
-    warn Dumper($rf);
+    Dwarn $rf;
   }
+
   unless($self->is_success(
-    $rf->{'Status'} eq '3DAUTH' ||
-    $rf->{'Status'} eq 'OK' ||
-    $rf->{'Status'} eq 'AUTHENTICATED' ||
-    $rf->{'Status'} eq 'REGISTERED' 
+    $rf->{'Status'} eq SAGEPAY_STATUS_3DSECURE ||
+    $rf->{'Status'} eq SAGEPAY_STATUS_PAYPAL_REDIRECT ||
+    $rf->{'Status'} eq SAGEPAY_STATUS_OK ||
+    $rf->{'Status'} eq SAGEPAY_STATUS_AUTHENTICATED ||
+    $rf->{'Status'} eq SAGEPAY_STATUS_REGISTERED 
     ? 1 : 0)) {
       my $code = substr $rf->{'StatusDetail'}, 0 ,4;
+
       if($ENV{'SAGEPAY_DEBUG_ERROR_ONLY'}) {
-        warn Dumper($rf);
+        Dwarn $rf;
       }
+
       $self->error_code($code);
-      $self->error_message($status->{$code} || 'There was an unknown problem taking your payment. Please try again');
+      $self->error_message($status->{$code} || $status->{UNKNOWN});
     }
 }
 
@@ -482,7 +664,7 @@ Business::OnlinePayment::SagePay - SagePay backend for Business::OnlinePayment
 
 =head1 VERSION
 
-version 0.12
+version 0.13
 
 =head1 SYNOPSIS
 
@@ -530,6 +712,31 @@ version 0.12
 
 This perl module provides integration with the SagePay VSP payments system.
 
+=head1 PAYPAL
+
+If the card type is set to C<PAYPAL> then when submitted the transaction will use SagePay's PayPal integration (see
+L<http://www.sagepay.com/products_services/paypal>). The URI to redirect the customer to after the page at
+PayPal has been completed should be set in the C<paypal_callback_uri> atrribute in C<content>.
+
+If C<result_code eq SAGEPAY_STATUS_PAYPAL_REDIRECT> then user should be redirected to
+the uri provided in method C<forward_to>.
+
+=head1 METHODS
+
+=head2 submit_paypal
+
+This method submits a COMPLETE transaction to SagePay to complete a PayPal transaction. C<authentication_id>
+amd C<amount> should be set in C<content>.
+
+=head2 SAGEPAY_STATUS_PAYPAL_REDIRECT
+
+    if ($tx->status_code eq $tx->SAGEPAY_STATUS_PAYPAL_REDIRECT) {
+        # redirct to $tx->forward_to ...
+    }
+
+Status to check if transaction result_code requires a redirect to PayPal. Can be called as a class or
+object method for convenience.
+
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-business-onlinepayment-sagepay at rt.cpan.org>, or through
@@ -570,6 +777,8 @@ L<Business::OnlinePayment>
 =head1 AUTHOR
 
   purge: Simon Elliott <cpan@browsing.co.uk>
+  
+  cubabit: Pete Smith
 
 =head1 ACKNOWLEDGEMENTS
 
@@ -583,4 +792,4 @@ L<Business::OnlinePayment>
 
 =cut
 
-1;
\ No newline at end of file
+1;
